@@ -642,7 +642,7 @@ dft_near2far fields::add_dft_near2far(const volume_list *where, const double *fr
 }
 
 //Modified from farfield_lowlevel
-std::vector<struct sourcedata> dft_near2far::near_sourcedata(const vec &x_0, double* farpt_list, size_t nfar_pts, std::complex<double>* dJ) {
+std::vector<struct sourcedata> dft_near2far::near_sourcedata(const vec &x_0, double* farpt_list, size_t nfar_pts, const std::complex<double>* dJ) {
   if (x_0.dim != D3 && x_0.dim != D2 && x_0.dim != Dcyl)
     meep::abort("only 2d or 3d or cylindrical far-field computation is supported");
   greenfunc green = x_0.dim == D2 ? green2d : green3d;
@@ -697,7 +697,7 @@ std::vector<struct sourcedata> dft_near2far::near_sourcedata(const vec &x_0, dou
           EH0 *= -1;
         EH0 /= f->S.multiplicity(ix0);
         if (x_0.dim == Dcyl){
-          EH0 /= xs.r();
+          if (xs.r() != 0) EH0 /= xs.r();
           //Somehow, a factor of (2pi)r for r of the near2far region was double counted.
           //2pi is canceled out by the integral over design region, where an extra factor of 2pi*r' (r' of the design region) is needed. See meepgeom.cpp
         }
